//+------------------------------------------------------------------+
//|                                                        Modes.mqh |
//|                                           Copyright 2014, denkir |
//|                           https://login.mql5.com/ru/users/denkir |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, denkir"
#property link      "https://login.mql5.com/ru/users/denkir"
//---
#include <Trade\Trade.mqh>
#include <Indicators\Oscilators.mqh>
#include <ChartObjects\ChartObjectsLines.mqh>
#include <ChartObjects\ChartObjectsShapes.mqh>
#include <Charts\Chart.mqh>
#include "CisNewBar.mqh"
//+------------------------------------------------------------------+
//| MQL-                                                        |
//+------------------------------------------------------------------+
enum ENUM_MQL_MODE
  {
   MQL_MODE_RELEASE=0,       // 
   MQL_MODE_DEBUG=1,         // 
   MQL_MODE_PROFILER=2,      //   
   MQL_MODE_TESTER=3,        // 
   MQL_MODE_OPTIMIZATION=4,  // 
   MQL_MODE_VISUAL=5,        //   
   MQL_MODE_FRAME=6,         // 
  };
//+------------------------------------------------------------------+
//| Class CModeBase                                                  |
//| Purpose: a base class for MQL-modes                              |            
//+------------------------------------------------------------------+
class CModeBase
  {
   //--- === Data members === --- 
private:
   //--- a macd object & values
   CiMACD            m_macd_obj;
   double            m_macd_main_vals[2];
   double            m_macd_sig_vals[2];

protected:
   long              m_pos_id;
   bool              m_is_new_bar;
   uint              m_trailing_stop;
   uint              m_trail_step;
   //--- trade objects
   CSymbolInfo       m_symbol_info;
   CTrade            m_trade;
   CPositionInfo     m_pos_info;
   CDealInfo         m_deal_info;
   //--- mql mode
   ENUM_MQL_MODE     m_mql_mode;

   //--- a new bar object
   CisNewBar         m_new_bar;
   //--- current tick signal flag
   bool              m_is_curr_tick_signal;
   //--- close order type
   ENUM_ORDER_TYPE   m_close_ord_type;

   //--- === Methods === --- 
public:
   //--- constructor/destructor
   void              CModeBase();
   void             ~CModeBase(void){};
   //--- initialization
   virtual bool      Init(int _fast_ema,int slow_ema,int _sig,ENUM_APPLIED_PRICE _app_price);
   virtual void      Deinit(void){};

   //--- Modules
   virtual void      Main(void){};

   //--- Procedures
   virtual void      Open(void){};
   virtual void      Close(void){};
   virtual void      Trail(void){};

   //--- Service
   static ENUM_MQL_MODE CheckMqlMode(void);
   ENUM_MQL_MODE     GetMqlMode(void);
   void              SetMqlMode(const ENUM_MQL_MODE _mode);
   //---
   void              SetTrailing(const uint _trailing,const uint _trail_step);

protected:
   //--- Functions
   ENUM_ORDER_TYPE   CheckOpenSignal(const ENUM_ORDER_TYPE _open_sig);
   ENUM_ORDER_TYPE   CheckCloseSignal(const ENUM_ORDER_TYPE _close_sig);
   ENUM_ORDER_TYPE   CheckTrailSignal(const ENUM_ORDER_TYPE _trail_sig,double &_sl_pr);
   //---
   double            GetMacdVal(const int _idx,const bool _is_main=true);

private:
   //--- Macros
   bool              RefreshIndicatorData(void);
   //--- normalization
   double            NormalPrice(double d);
   double            NormalDbl(double d,int n=-1);
   double            NormalSL(const ENUM_ORDER_TYPE _ord_type,double op,double pr,
                              uint SL,double stop);
   double            NormalTP(const ENUM_ORDER_TYPE _ord_type,double op,double pr,
                              uint _TP,double stop);
   double            NormalLot(const double _lot);
  };
//+------------------------------------------------------------------+
//| Default constructor                                              |
//+------------------------------------------------------------------+
void CModeBase::CModeBase(void)
  {
   this.m_mql_mode=WRONG_VALUE;
   this.m_is_new_bar=false;
   this.m_trailing_stop=0;
   ArrayInitialize(this.m_macd_main_vals,WRONG_VALUE);
   ArrayInitialize(this.m_macd_sig_vals,WRONG_VALUE);
  }
//+------------------------------------------------------------------+
//| Initialization                                                   |
//+------------------------------------------------------------------+
bool CModeBase::Init(int _fast_ema,int _slow_ema,int _sig,ENUM_APPLIED_PRICE _app_price)
  {
   if(this.m_symbol_info.Name(_Symbol))
      if(this.m_macd_obj.Create(_Symbol,_Period,_fast_ema,_slow_ema,_sig,_app_price))
        {
         this.m_is_curr_tick_signal=false;
         this.m_close_ord_type=WRONG_VALUE;
         this.m_mql_mode=this.CheckMqlMode();
         return true;
        }
//---
   return false;
  }
//+------------------------------------------------------------------+
//| Check a signal for opening                                       |
//+------------------------------------------------------------------+
ENUM_ORDER_TYPE CModeBase::CheckOpenSignal(const ENUM_ORDER_TYPE _open_sig)
  {
   ENUM_ORDER_TYPE curr_sig=WRONG_VALUE;

//---   
   if(this.RefreshIndicatorData())
     {
      //--- macd values
      double prev_main=this.m_macd_main_vals[0];
      double last_main=this.m_macd_main_vals[1];
      double prev_sig=this.m_macd_sig_vals[0];
      double last_sig=this.m_macd_sig_vals[1];
      //--- buy condition
      if(_open_sig==ORDER_TYPE_BUY)
        {
         if(prev_main<0.0 && last_main<0.0)
            if(prev_sig<0.0 && last_sig<0.0)
               if(prev_main<prev_sig && last_main>last_sig)
                  curr_sig=_open_sig;
        }
      //--- sell condition
      else if(_open_sig==ORDER_TYPE_SELL)
        {
         if(prev_main>0.0 && last_main>0.0)
            if(prev_sig>0.0 && last_sig>0.0)
               if(prev_main>prev_sig && last_main<last_sig)
                  curr_sig=_open_sig;
        }
     }
//---
   return curr_sig;
  }
//+------------------------------------------------------------------+
//| Check a signal for closing                                       |
//+------------------------------------------------------------------+
ENUM_ORDER_TYPE CModeBase::CheckCloseSignal(const ENUM_ORDER_TYPE _close_sig)
  {
   ENUM_ORDER_TYPE curr_sig=WRONG_VALUE;

//---   
   if(this.RefreshIndicatorData())
     {
      //--- macd values
      double prev_main=this.m_macd_main_vals[0];
      double last_main=this.m_macd_main_vals[1];
      double prev_sig=this.m_macd_sig_vals[0];
      double last_sig=this.m_macd_sig_vals[1];
      //--- buy condition
      if(_close_sig==ORDER_TYPE_BUY)
        {
         if(prev_main>0.0 && last_main>0.0)
            if(prev_sig>0.0 && last_sig>0.0)
               if(prev_main>prev_sig && last_main<last_sig)
                  curr_sig=_close_sig;
        }
      //--- sell condition
      else if(_close_sig==ORDER_TYPE_SELL)
        {
         if(prev_main<0.0 && last_main<0.0)
            if(prev_sig<0.0 && last_sig<0.0)
               if(prev_main<prev_sig && last_main>last_sig)
                  curr_sig=_close_sig;
        }
     }
//---
   return curr_sig;
  }
//+------------------------------------------------------------------+
//| Check a signal for trailing                                      |                                                                 
//+------------------------------------------------------------------+   
ENUM_ORDER_TYPE CModeBase::CheckTrailSignal(const ENUM_ORDER_TYPE _trail_sig,double &_sl_pr)
  {
   ENUM_ORDER_TYPE curr_ord_type=WRONG_VALUE;
   _sl_pr=0.0;
//---- get prices
   double pos_open_pr,pos_sl_pr,pos_tp_pr,apr;
   pos_open_pr=pos_sl_pr=pos_tp_pr=apr=WRONG_VALUE;
//---
   pos_open_pr=this.NormalDbl(this.m_pos_info.PriceOpen());
   pos_sl_pr=this.NormalDbl(this.m_pos_info.StopLoss());
   pos_tp_pr=this.NormalDbl(this.m_pos_info.TakeProfit());
//---
   if(this.m_symbol_info.RefreshRates())
     {
      apr=(_trail_sig==ORDER_TYPE_BUY)? this.m_symbol_info.Bid():
          this.m_symbol_info.Ask();
      double curr_sl_pr;
      int curr_StopLevel=this.m_symbol_info.StopsLevel();
      double StopLevel_points=this.NormalDbl(_Point*curr_StopLevel);
      //---
      if(this.m_trailing_stop>0)
        {
         //---   SL     
         curr_sl_pr=this.NormalSL(_trail_sig,apr,apr,this.m_trailing_stop,StopLevel_points);
         //--- refer to order type
         double trailing_stop_point=this.NormalDbl(_Point*this.m_trailing_stop);
         double trail_step_point=this.NormalDbl(_Point*this.m_trail_step);
         bool IsToModify=false;
         //---     
         if(_trail_sig==ORDER_TYPE_BUY)
           {
            //---     
            if(apr-pos_open_pr>=trailing_stop_point)
              {
               //---    -   
               if(curr_sl_pr>pos_open_pr)
                  //---    -    -                    
                  if(curr_sl_pr>(pos_sl_pr+trail_step_point) || pos_sl_pr==this.NormalPrice(0)) // todo
                     IsToModify=true;
              }
           }
         //---     
         else if(_trail_sig==ORDER_TYPE_SELL)
           {
            //---      
            if(pos_open_pr-apr>=(trailing_stop_point+trail_step_point))
              {
               //---    -   
               if(curr_sl_pr<pos_open_pr)
                  //---    -    -                    
                  if(curr_sl_pr<(pos_sl_pr-trail_step_point) || pos_sl_pr==this.NormalPrice(0)) // todo
                     IsToModify=true;
              }
           }
         //---   
         if(IsToModify)
           {
            curr_ord_type=_trail_sig;
            _sl_pr=curr_sl_pr;
           }
        }
     }
//---
   return curr_ord_type;
  }
//+------------------------------------------------------------------+
//|   MACD                                           |
//+------------------------------------------------------------------+ 
double CModeBase::GetMacdVal(const int _idx,const bool _is_main=true)
  {
   if(_idx>=ArraySize(this.m_macd_main_vals))
      return WRONG_VALUE;
//---
   if(_is_main)
      return this.m_macd_main_vals[_idx];
   else
      return this.m_macd_sig_vals[_idx];
  }
//+------------------------------------------------------------------+
//| Check the MQL-mode                                               |
//+------------------------------------------------------------------+
static ENUM_MQL_MODE CModeBase::CheckMqlMode(void)
  {
   ENUM_MQL_MODE curr_mode=WRONG_VALUE;

//---   
   if(MQLInfoInteger(MQL_DEBUG))
      curr_mode=MQL_MODE_DEBUG;
//---    
   else if(MQLInfoInteger(MQL_PROFILER))
      curr_mode=MQL_MODE_PROFILER;
//---    
   else if(MQLInfoInteger(MQL_VISUAL_MODE))
      curr_mode=MQL_MODE_VISUAL;
//---   
   else if(MQLInfoInteger(MQL_OPTIMIZATION))
      curr_mode=MQL_MODE_OPTIMIZATION;
//---   
   else if(MQLInfoInteger(MQL_TESTER))
      curr_mode=MQL_MODE_TESTER;
//---      
   else if(MQLInfoInteger(MQL_FRAME_MODE))
      curr_mode=MQL_MODE_FRAME;
//---  -
   else
      curr_mode=MQL_MODE_RELEASE;
//---
   return curr_mode;
  }
//+------------------------------------------------------------------+
//| Get the MQL-mode                                                 |
//+------------------------------------------------------------------+
ENUM_MQL_MODE CModeBase::GetMqlMode(void)
  {
   return this.m_mql_mode;
  }
//+------------------------------------------------------------------+
//| Set the MQL-mode                                                 |
//+------------------------------------------------------------------+
void CModeBase::SetMqlMode(const ENUM_MQL_MODE _mode)
  {
   this.m_mql_mode=_mode;
  }
//+------------------------------------------------------------------+
//| Set the trailing stop                                            |
//+------------------------------------------------------------------+
void CModeBase::SetTrailing(const uint _trailing,const uint _trail_step)
  {
   this.m_trailing_stop=_trailing;
   this.m_trail_step=_trail_step;
  };
//+------------------------------------------------------------------+
//| Refresh indicator data                                           |
//+------------------------------------------------------------------+
bool CModeBase::RefreshIndicatorData(void)
  {
   double main_vals[2]; // main_vals[0] - previous, main_vals[1] - last
   double sig_vals[2];
//---   
   this.m_macd_obj.Refresh(-1);
   for(int idx=ArraySize(main_vals),jdx=0;idx>=1;idx--,jdx++)
     {
      main_vals[jdx]=this.m_macd_obj.Main(idx);
      sig_vals[jdx]=this.m_macd_obj.Signal(idx);
      //---
      if(main_vals[jdx]==EMPTY_VALUE || sig_vals[jdx]==EMPTY_VALUE)
         return false;
      else
        {
         this.m_macd_main_vals[jdx]=main_vals[jdx];
         this.m_macd_sig_vals[jdx]=sig_vals[jdx];
        }
     }
//---
   return true;
  }
//+------------------------------------------------------------------+
//| Normal value                                                     |
//+------------------------------------------------------------------+
double CModeBase::NormalDbl(const double _val,int n=-1)
  {
   double norm_val=WRONG_VALUE;
//---    
   if(n<0)
      norm_val=NormalizeDouble(_val,_Digits);
   else
      norm_val=NormalizeDouble(_val,n);
//----     
   return norm_val;
  }
//+------------------------------------------------------------------+
//| Normal price                                                     |
//+------------------------------------------------------------------+
double CModeBase::NormalPrice(double _pr)
  {
   double norml_pr=WRONG_VALUE;
//---   
   double tick_size=this.m_symbol_info.TickSize();
   if(tick_size>0.0)
      norml_pr=this.NormalDbl(MathRound(_pr/tick_size)*tick_size);
//----   
   return norml_pr;
  }
//+------------------------------------------------------------------+
//| Normalize take profit                                            |
//+------------------------------------------------------------------+
double CModeBase::NormalTP(const ENUM_ORDER_TYPE _ord_type,double op,double pr,uint _TP,double stop)
  {
   if(_TP==0)
      return(this.NormalPrice(0));
//---
   if(_ord_type==ORDER_TYPE_BUY)
      return(this.NormalPrice(MathMax(op+_TP*_Point,pr+stop)));
   if(_ord_type==ORDER_TYPE_SELL)
      return(this.NormalPrice(MathMin(op-_TP*_Point,pr-stop)));
//---- 
   return WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Normalize stop loss                                              |
//+------------------------------------------------------------------+
double CModeBase::NormalSL(const ENUM_ORDER_TYPE _ord_type,double op,double pr,
                           uint _SL,double stop)
  {
   if(_SL==0)
      return this.NormalPrice(0);
//---  
   if(_ord_type==ORDER_TYPE_BUY)
      return this.NormalPrice(MathMin(op-_SL*_Point,pr-stop));
//---
   if(_ord_type==ORDER_TYPE_SELL)
      return this.NormalPrice(MathMax(op+_SL*_Point,pr+stop));
//----
   return WRONG_VALUE;
  }
//+------------------------------------------------------------------+
//| Lot normalization by symbol properties                           |
//+------------------------------------------------------------------+
double CModeBase::NormalLot(const double _lot)
  {
   double ll=WRONG_VALUE;
//---
   int k=0;
   ll=_lot;
   double ls=this.m_symbol_info.LotsStep();
   if(ls<=0.001) k=3;
   else if(ls<=0.01)
               k=2;
   else if(ls<=0.1) k=1;
//---
   ll=this.NormalDbl(MathMin(this.m_symbol_info.LotsMax(),
                     MathMax(this.m_symbol_info.LotsMin(),ll)),k);
//----
   return ll;
  }
//+------------------------------------------------------------------+
//| Class CModeTester                                                |
//| Purpose: a class for the tester mode                             |            
//| Derives from class CModeBase.                                    |
//+------------------------------------------------------------------+
class CModeTester : public CModeBase
  {
   //--- === Data members === --- 
protected:
   uint              m_start_cnt;

   //--- === Methods === --- 
public:
   //--- constructor/destructor
   void              CModeTester(void){};
   void             ~CModeTester(void){};

   //--- Modules
   virtual void      Main(void);

   //--- Procedures
   virtual void      Open(void);
   virtual void      Close(void);
   virtual void      Trail(void);
  };
//+------------------------------------------------------------------+
//|                                                     |
//+------------------------------------------------------------------+
void CModeTester::Main(void)
  {
//--- 1) 
   this.Close();
//--- 2) 
   this.Open();
//--- 3) 
   this.Trail();
  }
//+------------------------------------------------------------------+
//| Open module                                                      |
//+------------------------------------------------------------------+
void CModeTester::Open(void)
  {
   this.m_is_new_bar=this.m_new_bar.isNewBar();

//---    
   if(this.m_is_new_bar)
      if(!this.m_pos_info.Select(_Symbol))
         for(ENUM_ORDER_TYPE type_idx=ORDER_TYPE_BUY;type_idx<=ORDER_TYPE_SELL;type_idx++)
           {
            ENUM_ORDER_TYPE curr_signal=this.CheckOpenSignal(type_idx);
            //---     
            if(curr_signal==type_idx)
              {
               double prev_main=this.GetMacdVal(0);
               double last_main=this.GetMacdVal(1);
               double prev_sig=this.GetMacdVal(0,false);
               double last_sig=this.GetMacdVal(1,false);
               //--- 
               PrintFormat("\n---===   : %s===---",StringSubstr(EnumToString(curr_signal),11));
               PrintFormat(" , : %0.6f; : %0.6f",prev_main,prev_sig);
               PrintFormat(" , : %0.6f; : %0.6f",last_main,last_sig);

               //---                 
               if(this.m_symbol_info.RefreshRates())
                 {
                  //---   
                  double curr_price=WRONG_VALUE;
                  if(curr_signal==ORDER_TYPE_BUY)
                     curr_price=this.m_symbol_info.Ask();
                  else if(ORDER_TYPE_SELL)
                     curr_price=this.m_symbol_info.Bid();
                  //---  
                  if(this.m_trade.PositionOpen(_Symbol,curr_signal,0.03,curr_price,0.0,0.0))
                    {
                     if(this.m_pos_info.Select(_Symbol))
                        this.m_pos_id=this.m_pos_info.Identifier();
                     this.m_is_curr_tick_signal=true;
                     break;
                    }
                 }
              }
           }
  }
//+------------------------------------------------------------------+
//| Close module                                                     |
//+------------------------------------------------------------------+
void CModeTester::Close(void)
  {
//---  
   if(this.m_is_curr_tick_signal)
      this.m_is_curr_tick_signal=false;

   ENUM_ORDER_TYPE deal_ord_type=WRONG_VALUE;
//--- ,      -
   if(!this.m_pos_info.Select(_Symbol))
     {
      if(this.m_pos_id>0)
        {
         //---    
         if(HistorySelectByPosition(this.m_pos_id))
           {
            int pos_deals=HistoryDealsTotal();
            //---
            for(int idx=pos_deals-1;idx>=0;idx--)
               if(this.m_deal_info.SelectByIndex(idx))
                 {
                  ENUM_DEAL_ENTRY deal_entry_type=this.m_deal_info.Entry();
                  //---   
                  if(deal_entry_type==DEAL_ENTRY_IN)
                    {
                     this.m_close_ord_type=(ENUM_ORDER_TYPE)this.m_deal_info.DealType();
                     this.m_pos_id=0;
                     PrintFormat("\n---===     -: %s===---",
                                 StringSubstr(EnumToString(this.m_close_ord_type),11));
                     //---
                     break;
                    }
                 }
           }
        }
     }
//---        
   else
     {
      if(this.m_is_new_bar)
         for(ENUM_ORDER_TYPE type_idx=ORDER_TYPE_BUY;type_idx<=ORDER_TYPE_SELL;type_idx++)
           {
            ENUM_ORDER_TYPE curr_pos_type=(ENUM_ORDER_TYPE)this.m_pos_info.PositionType();
            if(curr_pos_type==type_idx)
              {
               ENUM_ORDER_TYPE curr_signal=this.CheckCloseSignal(type_idx);
               //---     
               if(curr_signal==type_idx)
                 {
                  double prev_main=this.GetMacdVal(0);
                  double last_main=this.GetMacdVal(1);
                  double prev_sig=this.GetMacdVal(0,false);
                  double last_sig=this.GetMacdVal(1,false);

                  PrintFormat("\n---===   : %s===---",StringSubstr(EnumToString(curr_signal),11));
                  PrintFormat(" , : %0.6f; : %0.6f",prev_main,prev_sig);
                  PrintFormat(" , : %0.6f; : %0.6f",last_main,last_sig);
                  this.m_close_ord_type=curr_signal;
                  //---  
                  if(this.m_trade.PositionClose(_Symbol))
                    {
                     this.m_pos_id=0;
                     break;
                    }
                 }
              }
           }
     }
  }
//+------------------------------------------------------------------+
//| Trailing module                                                  |
//+------------------------------------------------------------------+
void CModeTester::Trail(void)
  {
//---    ,  
   if(!this.m_trailing_stop)
      return;

//---     ,   
   if(this.m_is_curr_tick_signal)
      return;

//---    
   if(this.m_pos_info.Select(_Symbol))
      for(ENUM_ORDER_TYPE type_idx=ORDER_TYPE_BUY;type_idx<=ORDER_TYPE_SELL;type_idx++)
        {
         ENUM_ORDER_TYPE curr_pos_type=(ENUM_ORDER_TYPE)this.m_pos_info.PositionType();
         if(curr_pos_type==type_idx)
           {

            double curr_sl_pr;
            double curr_tp_pr=this.m_pos_info.TakeProfit();
            ENUM_ORDER_TYPE curr_signal=this.CheckTrailSignal(type_idx,curr_sl_pr);
            //---     
            if(curr_signal!=WRONG_VALUE)
               if(curr_signal==type_idx)
                 {
                  //---  
                  if(this.m_trade.PositionModify(_Symbol,curr_sl_pr,curr_tp_pr))
                     break;
                 }
           }
        }
  }
//+------------------------------------------------------------------+
//| Class CModeVisual                                                |
//| Purpose: a class for the tester mode                             |            
//| Derives from class CModeBase.                                    |
//+------------------------------------------------------------------+
class CModeVisual : public CModeTester
  {
   //--- === Data members === --- 
private:
   CArrayObj         m_objects_arr;
   double            m_subwindow_max;
   double            m_subwindow_min;
   //--- === Methods === --- 
public:
   //--- constructor/destructor
   void              CModeVisual(void);
   void             ~CModeVisual(void);

   //--- Procedures
   virtual void      Open(void);
   virtual void      Close(void);

private:
   bool              CreateSignalLine(const bool _is_open_sig,const bool _is_new_bar=true);
   bool              CreateRectangle(const ENUM_ORDER_TYPE _signal);
   void              RefreshRectangles(void);
  };
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+
void CModeVisual::CModeVisual(void)
  {
   this.m_objects_arr.FreeMode(false);
   this.m_trade.LogLevel(0);
  };
//+------------------------------------------------------------------+
//|                                                        |
//+------------------------------------------------------------------+
void CModeVisual::~CModeVisual(void)
  {

  };
//+------------------------------------------------------------------+
//| Open module                                                      |
//+------------------------------------------------------------------+
void CModeVisual::Open(void)
  {
//---    
   CModeTester::Open();

//--- id    
   static long prev_id;

//---   
   if(prev_id==0)
      //---       
      if(this.m_pos_id>0)
        {
         //--- 
         if(!this.CreateSignalLine())
            Print("     !");
        }
//---  id 
   if(prev_id!=this.m_pos_id)
      prev_id=this.m_pos_id;
  }
//+------------------------------------------------------------------+
//| Close module                                                     |
//+------------------------------------------------------------------+
void CModeVisual::Close(void)
  {
//---  
   this.RefreshRectangles();

//---    
   CModeTester::Close();

//--- id    
   static long prev_id;

//---  
   if(prev_id>0)
      //---      
      if(this.m_pos_id==0)
        {
         //--- 
         if(!this.CreateSignalLine(false,this.m_is_new_bar))
            Print("     !");
         //--- 
         if(this.m_close_ord_type>WRONG_VALUE)
           {
            if(!this.CreateRectangle(this.m_close_ord_type))
               Print("  !");
            //---
            this.m_close_ord_type=WRONG_VALUE;
           }
        }
//---  id 
   if(prev_id!=this.m_pos_id)
      prev_id=this.m_pos_id;
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
bool CModeVisual::CreateSignalLine(const bool _is_open_sig=true,const bool _is_new_bar=true)
  {
   string line_name="Open_signal_line";
   color line_clr=clrRed;
//---
   if(!_is_open_sig)
     {
      line_clr=clrBlue;
      line_name="Close_signal_line";
     }
   line_name+=IntegerToString(this.m_objects_arr.Total()/3+1);

//--- 
   CChartObjectVLine *ptr_open_line=new CChartObjectVLine;
//---
   datetime line_time=this.m_new_bar.GetLastBarTime();
   if(!_is_new_bar)
      line_time=TimeCurrent();
//---
   if(CheckPointer(ptr_open_line)==POINTER_DYNAMIC)
      if(ptr_open_line.Create(0,line_name,0,line_time))
         //--- 
         if(ptr_open_line.Color(line_clr))
            //--- 
            if(this.m_objects_arr.Add(ptr_open_line))
              {
               ChartRedraw();
               //---
               return true;
              }
//---
   return false;
  }
//+------------------------------------------------------------------+
//|                                              |
//+------------------------------------------------------------------+
bool CModeVisual::CreateRectangle(const ENUM_ORDER_TYPE _signal)
  {
//--- 
   CChartObjectRectangle *ptr_rect=new CChartObjectRectangle;
   string rect_name="Signal_rect";
   rect_name+=IntegerToString(this.m_objects_arr.Total()/3+1);
   color rect_clr=clrLightBlue;
   if(_signal==ORDER_TYPE_SELL)
      rect_clr=clrLightPink;
//---
   datetime dates[2];
   ArrayInitialize(dates,0);
//---
   for(int i=0;i<ArraySize(dates);i++)
     {
      CChartObjectVLine *ptr_open_line=this.m_objects_arr.At
                                       (this.m_objects_arr.Total()-2+i);
      dates[i]=ptr_open_line.Time(0);
     }
//---
   ResetLastError();
//---  
   if(!ptr_rect.Create(0,rect_name,1,dates[0],this.m_subwindow_max,dates[1],this.m_subwindow_min))
     {
      PrintFormat("   , : %d",GetLastError());
      return false;
     }
   else
     {
      ptr_rect.Color(rect_clr);
      ptr_rect.SetInteger(OBJPROP_FILL,true);
      //---      
      ChartRedraw();
     }
//---
   if(!this.m_objects_arr.Add(ptr_rect))
      Print("  !");
//---
   return true;
  }
//+------------------------------------------------------------------+
//|                                            |
//+------------------------------------------------------------------+
void CModeVisual::RefreshRectangles(void)
  {
//--- 
   CChart myChart;
   myChart.Attach();
   double pr_max=myChart.PriceMax(1);
   double pr_min=myChart.PriceMin(1);
//---  \ 
// static double subwindow_max,subwindow_min;
   bool is_to_move=false;
   if(this.m_subwindow_max<pr_max)
     {
      is_to_move=true;
      this.m_subwindow_max=pr_max;
     }
   if(this.m_subwindow_min>pr_min)
     {
      is_to_move=true;
      this.m_subwindow_min=pr_min;
     }
//---    
   if(is_to_move)
     {
      double prices[2];
      prices[0]=this.m_subwindow_max;
      prices[1]=this.m_subwindow_min;
      //---  
      for(int jdx=0;jdx<this.m_objects_arr.Total();jdx++)
        {
         CChartObject *ptr_curr_rect=this.m_objects_arr.At(jdx);
         if(ptr_curr_rect!=NULL)
            if(ptr_curr_rect.NumPoints()==2)
               for(int point_idx=0;point_idx<2;point_idx++)
                 {
                  datetime curr_time=ptr_curr_rect.Time(point_idx);
                  ptr_curr_rect.SetPoint(point_idx,curr_time,prices[point_idx]);
                 }
        }
      //---
      ChartRedraw();
     }
  }
  
//--- [EOF]
